﻿using Microsoft.EntityFrameworkCore;
using Senlan.VersionApp.Data;
using System.Linq.Expressions;

namespace Senlan.VersionApp.Services
{
    public class BaseService<T> : IBaseService<T> where T : class
    {
        public readonly SenlanDbContext db;
        private readonly HttpContext httpContext;

#pragma warning disable
        public BaseService(SenlanDbContext db)
        {
            this.db = db;
        }
        public BaseService(SenlanDbContext db, IHttpContextAccessor contextAccessor)
        {
            this.db = db;
            this.httpContext = contextAccessor.HttpContext;
        }


        public async Task<T> Add(T entity)
        {
            await db.Set<T>().AddAsync(entity);
            await db.SaveChangesAsync();
            return entity;
        }

        public async Task<bool> Add(List<T> entities)
        {
            await db.Set<T>().AddRangeAsync(entities);
            return await db.SaveChangesAsync() > 0;
        }

        public async Task<bool> Delete(T entity)
        {
            db.Set<T>().Remove(entity);
            return await db.SaveChangesAsync() > 0;
        }

        public async Task<bool> Delete(Guid id)
        {
            var entity = await db.Set<T>().FindAsync(id);
            if (entity != null)
            {
                db.Set<T>().Remove(entity);
            }
            return await db.SaveChangesAsync() > 0;
        }

        async Task<bool> IBaseService<T>.Delete(Expression<Func<T, bool>> whereLambda)
        {
            var entities = await db.Set<T>().Where(whereLambda).ToListAsync();
            db.Set<T>().RemoveRange(entities);
            return await db.SaveChangesAsync() > 0;
        }

        public async Task<T?> Get(Guid id)
        {
            return await db.Set<T>().FindAsync(id);
        }

        public async Task<T?> Get(Expression<Func<T, bool>> whereLambda)
        {
            return await db.Set<T>().AsNoTracking().Where(whereLambda).FirstOrDefaultAsync();
        }

        public async Task<List<T>> GetList(Expression<Func<T, bool>> whereLambda)
        {
            return await db.Set<T>().AsNoTracking().Where(whereLambda).ToListAsync();
        }



        public async Task<bool> IsExist(Expression<Func<T, bool>> whereLambda)
        {
            return await db.Set<T>().AnyAsync(whereLambda);
        }

        public async Task<Tuple<List<T>, int>> GetPaged<S>(int pageSize, int pageIndex, Expression<Func<T, bool>> whereLambda, Expression<Func<T, S>> orderByLambda, bool isAsc)
        {
            var query = db.Set<T>().Where(whereLambda);

            var total = await query.CountAsync();

            if (isAsc)
            {
                query = query.OrderBy(orderByLambda);
            }
            else
            {
                query = query.OrderByDescending(orderByLambda);
            }

            var entities = await query.Skip(pageSize * (pageIndex - 1))
                                      .Take(pageSize).ToListAsync();
            return new Tuple<List<T>, int>(entities, total);
        }

        public async Task<bool> Update(T entity, List<string> updatePropertyList)
        {

            db.Attach(entity);
            var entry = db.Entry(entity);
            if (updatePropertyList == null)
            {
                entry.State = EntityState.Modified;//全字段更新
            }
            else
            {
                updatePropertyList.ForEach(c =>
                {
                    entry.Property(c).IsModified = true; //部分字段更新的写法
                });

            }
            return await db.SaveChangesAsync() > 0;
        }

        public async Task<bool> Update(List<T> entitys)
        {
            entitys.ForEach(c =>
            {
                db.Attach(c);
                db.Entry(c).State = EntityState.Modified;
            });

            return await db.SaveChangesAsync() > 0;
        }
    }
}
